/*********************************************/
/*       File name:  session.c               */
/*    Project name:  socket/tag              */
/*      Month/Year:  Jan/2003                */
/*          Author:  Shuichi YUKITA          */
/*********************************************/

#include <sys/types.h>
#include <curses.h>
#include <signal.h>

#define BUF_LEN 20
#define MIN_X	1
#define MIN_Y	1
#define MAX_X	60
#define MAX_Y	20

/* ړL[̊蓖 */
#define NORTH 'k'
#define SOUTH 'j'
#define EAST  'l'
#define WEST  'h'
#define QUIT  'q'

/* vCx[gϐ */
static  int    session_soc;     /* socket  */
static  fd_set mask;            /* fd mask */
static  int    width;           /* width of the mask */
static  char   my_char, peer_char; /* character */

typedef	struct{
	int	x,y; /* current position */
	char	look;  /* character */
} PLAYER;

static PLAYER me, peer;    /* Ƒ̏Ԃێϐ  */

static char buf[BUF_LEN];  /* Mpobt@ */

static WINDOW *win;	   /* \pEBhE */

/*  session W[ɃvCx[gȊ֐  */
static void hide(PLAYER *who);
static void show(PLAYER *who);
static int  update(PLAYER *who, int c);
static int  interpret(PLAYER *who);
static void die();

/*  session W[̏  */
void session_init(int soc,
                  char mc, int mx, int my,
                  char pc, int px, int py)
	   /* soc  \Pbg̃fBXNv^ */
	   /* mc   ̕\p̕       */
	   /* mx   ̏xW          */
	   /* my   ̏yW          */
	   /* pc   ̕\p̕       */
	   /* px   ̏xW          */
	   /* py   ̏yW          */
{
/* f[^̐ݒ */
	session_soc = soc;
	width = soc + 1;
	FD_ZERO(&mask);
	FD_SET(0, &mask);
	FD_SET(session_soc, &mask);
	me.look = mc;
	peer.look = pc;

	me.x = mx;  me.y = my;
	peer.x = px;  peer.y = py;

	/* curses ̏ */
	initscr();
	signal(SIGINT, die);

	/* EBhEg */
	win = newwin(MAX_Y + 2, MAX_X + 2, 0, 0);
	box(win, '|', '-');

	/* [ݒ */
	cbreak();
	noecho();
}


/*  session C[v  */
void session_loop()
{
	int c;         /* L[{[h͕̓ */
	int flag;      /* [vptbO       */
	fd_set readOk; /* d͂̃}XN         */

	/* ̎p̕\ */
	show(&me);

	/* [vptbȌ */
	flag = 1;

	/* [v */
	while(1){
		/*  select ̑Oɖ readOk B */
		readOk = mask;
		select(width,(fd_set *)&readOk,NULL,NULL,NULL);

		/*  L[{[h̓͂肩H */
		if ( FD_ISSET(0, &readOk ) ){
			c = getchar();            /* L[{[h     */
			hide(&me);                /* ̎pB         */
			flag = update(&me, c);    /* ԂXV             */
			show(&me);                /* ̎p         */
			write(session_soc, buf, BUF_LEN); /* XVf[^𑗂 */
			if(flag == 0) break;      /* [vo     */
		}

		 /*  \PbgɃf[^肩H */
		if ( FD_ISSET(session_soc, &readOk ) ){
			read(session_soc,buf,BUF_LEN); /* \Pbg  */
			hide(&peer);              /* ̎pB         */
			flag = interpret(&peer);  /* ̏Ԃ̍XV       */
			show(&peer);              /* ̎p         */
			if(flag == 0) break;      /* [vo     */
		}

	}
	/* [𕜋ďI */
	die();
}


static void show(PLAYER *who)
	/* who  N\邩  */
{
	wmove(win, who->y, who->x); /* whȍꏊɓ           */
	waddch(win, who->look);     /* ̏ꏊwholooku */
	wmove(win, who->y, who->x); /* J[\߂            */
	wrefresh(win);              /* EBhE̍XV          */
}


static void hide(PLAYER *who)
	/* who  N̎p  */
{
	wmove(win, who->y, who->x); /* whȍꏊɓ           */
	waddch(win, ' ');           /* ̏ꏊɃXy[X  */
}


static int update(PLAYER *who, int c)
	/* who  N             */
	/* c    ǂ֓ */
{
	switch(c){
	case WEST:
		if ( who->x > MIN_X )
			who->x--;
		break;
	case SOUTH:
		if ( who->y < MAX_Y )
			who->y++;
		break;
	case NORTH:
		if ( who->y > MIN_X )
			who->y--;
		break;
	case EAST:
		if ( who->x < MAX_X )
			who->x++;
		break;
	case QUIT:
		buf[0]=QUIT;
		return 0;
	default:
		break;
	}

	/* XVꂽʒuf[^𑗐Mpɕϊ */
	sprintf(buf, "%d %d\n", who->x, who->y);
	return 1;
}

static int interpret(PLAYER *who)
	/* who ߌ̃f[^̊i[w */
{
	/* 肪I] */
	if (buf[0] == 'q')
		return 0;

	/* M񂩂ʒuf[^ɕϊB */
	sscanf(buf, "%d %d", &who->x, &who->y);
	return 1;
}

static void die()
{
	endwin();  /* [̕ */
	exit(0);
}
